#!/usr/bin/perl
###############################################################################
#                                                                             #
# IPFire.org - A linux based firewall                                         #
# Copyright (C) 2007  Michael Tremer & Christian Schmidt                      #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################
#
#
# This file is AddOn of the IPFire Firewall base on IPCop-Addon IPTraffic
#
# $Id: iptraffic.cgi, v0.8.1 2021-01-10 18:45:00 RadioCarbon <frank@cybermainzel.de>$
#

# Add entry in menu
# MENUENTRY status 065 "IPTraffic" "IPTraffic"

my $debug	= 0;
# enable only the following on debugging purpose
use warnings;
use strict;
# needs Perl-DBI-Addon
use DBI;
use Carp ();
use CGI::Carp 'fatalsToBrowser';
use Socket;
use Fatal qw/ open /;

require '/var/ipfire/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

my $version = 'v0.8';

my @line;
my @iptclients;
my @get_param;
my @devs_net;
my ( %ips, %arpcache, %iptsettings, %netsettings ) = ();

my $idarp = 0;
my $enabled_count = 0;
my $IPTrafficTable = "IPTraffic";

my $graphdir	= '/graphs/iptraffic';
my $imgsys		= "/images";
my $imgstatic	= "$imgsys/iptraffic";
my $logfile		= "/var/log/iptraffic/iptraffic";
my $rrdimgdir	= "/srv/web/ipfire/html/graphs/iptraffic";
my $rrddir		= "/var/log/rrd/iptraffic";
my $iptrafficdb	= "/var/log/iptraffic/iptraffic.db";

## some iptraffic settings
$iptsettings{'ACTION'} = '';
$iptsettings{'SORT'} = '';
$iptsettings{'PARAM'} = '';
$iptsettings{'CLIENTNAME'} = '';

my $contents = $ENV{'QUERY_STRING'};

&Header::getcgihash(\%iptsettings);
&General::readhash('/var/ipfire/ethernet/settings', \%netsettings);
#if (-e '/var/ipfire/addons/xtiface/xtiface.conf') {
#	&General::readhash('/var/ipfire/addons/xtiface/xtiface.conf', \%netsettings);
#}

# add client to list
if ($iptsettings{'ACTION'} eq 'add'){
	my $client = $iptsettings{'PARAM'};
	my $name = $iptsettings{'HOST'};
	setClientAction($client,'ADD',$name);
	siteReload();
}
# delete client from list
if ($iptsettings{'ACTION'} eq 'del'){
	my $client = $iptsettings{'PARAM'};

	setClientAction($client,'DEL');
	siteReload();
}
# set trafficlimit on
if ($iptsettings{'ACTION'} eq 'ON'){
	my $client = $iptsettings{'PARAM'};

	setClientAction($client,'ON');
#	siteReload();
}
# set trafficlimit off
if ($iptsettings{'ACTION'} eq 'OFF'){
	my $client = $iptsettings{'PARAM'};

	setClientAction($client,'OFF');
#	siteReload();
}
# set sort direction
if ($contents) {
	@get_param = split(/=/,$contents);
	$iptsettings{'SORT'} = $get_param[1];
}

&Header::showhttpheaders();
#&Header::openpage('IPTraffic', 1, '');
&Header::openpage($Lang::tr{'IPTraffic_title'}, 1, '');
&Header::openbigbox('100%', 'left');

#########################################################
# START DEBUG DEBUG
if ($debug) {
	&Header::openbox('100%', 'left', 'DEBUG');
	my $debugCount = 0;
	foreach my $line (sort keys %iptsettings) {
		print "$line = $iptsettings{$line}<br />\n";
		$debugCount++;
	}
	print "&nbsp;Count: $debugCount <br>\n";
	print "&nbsp;Contents: $contents<br>";
	print "&nbsp;Get-param: @get_param<br>";
	
	hrline();

	my $netdebug = 0;
	foreach (sort keys %netsettings) {
		print"$_ = $netsettings{$_}<br />\n";
		$netdebug++;
	}
	&Header::closebox();
}
# END DEBUG DEBUG
#########################################################

### Call funktions, default List with Arplist
if ($iptsettings{'ACTION'} eq 'GRAPH'){
	disp_singleclient();
}
else{
#	disp_config();
	disp_clientlist();
	disp_arp();
}

&Header::closebigbox();
&Header::closepage();

#########################################################
# subroutines

### reload self
sub siteReload {
	print "Status: 302 Found\n";
	print "Location: $ENV{'SCRIPT_NAME'}\n";
	print "URI: <$ENV{'SCRIPT_NAME'}>\n";
	print "Content-type: text/html\r\n\r\n";
}

### display config area
sub disp_config {
	my $title = 'Konfiguration';

	&Header::openbox('100%', 'left', "$title Titel");
		print <<END
<table width='100%' align='center' border="1">
	<tr>
		<td width="5%">&nbsp;</td>
		<td>Device to limit:</td>
		<td>Device max(MBit/sek):</td>
		<td>Device limit(kBit/sek):</td>
		<td width="15%">Version: $version</td>
	</tr>
</table>
END
;
	&Header::closebox();
}

### display single client graphs
sub disp_singleclient {
	my $client = $iptsettings{'PARAM'};
	my $tmpHost = getHostnameFromTable($client);
	my $host = length($tmpHost) > 1 ? $tmpHost : $client ;
	my $back = "<a href='$ENV{'SCRIPT_NAME'}'>";

	&Header::openbox('100%', 'center', "$Lang::tr{'IPTraffic_graphs_stat'}");
	print "
	$back<img src='$imgsys/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a>
	<hr />";

	if (-e "$rrdimgdir/$client-day.png") {
		print "
	<img src='$graphdir/$client-hour.png' border='0' alt='$host-$Lang::tr{'hour'}' />
	<img src='$graphdir/$client-day.png' border='0' alt='$host-$Lang::tr{'day'}' />
	<img src='$graphdir/$client-week.png' border='0' alt='$host-$Lang::tr{'week'}' />
	<img src='$graphdir/$client-month.png' border='0' alt='$host-$Lang::tr{'month'}' />
	<img src='$graphdir/$client-year.png' border='0' alt='$host-$Lang::tr{'year'}' />
	<hr />
	$back<img src='$imgsys/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a>
";
	}
	else {
		print "
<div width='100%' align='center'>
	<br>$rrdimgdir/$client-day.png $Lang::tr{'IPTraffic_not_found'}<br /><br />
	<b>$Lang::tr{'no information available'}</b><br />
</div>";
	}
	&Header::closebox();
}

sub disp_clientlist {
	my ($sortarrowip, $sortarrowhn, $sortarrowou, $sortarrowouM, $sortarrowouW, $sortarrowouD, $sortarrowin, $sortarrowinM, $sortarrowinW, $sortarrowinD);
	$sortarrowip = $sortarrowhn = $sortarrowou = $sortarrowouM = $sortarrowouW = $sortarrowouD = $sortarrowin = $sortarrowinM = $sortarrowinW = $sortarrowinD =  '&nbsp;';
	my ($sum_incoming_all, $sum_incoming_month, $sum_incoming_week, $sum_incoming_day) = 0;
	my ($sum_outgoing_all, $sum_outgoing_month, $sum_outgoing_week, $sum_outgoing_day) = 0;
	my ($ipsort, $hnsort, $insort, $ousort, $insortM, $ousortM, $insortW, $ousortW, $insortD, $ousortD) = '';
	my ($i, $count) = 0;
	my @sorted_ip = '';
	my %traffic = getAllTrafficFromTable();
	my @ips = getIPfromTable();
	splice(@ips, $i, 1) if ($ips[$i] eq '');
	my $sort = $iptsettings{'SORT'};
	chomp $sort;
	if (length($sort) < 1) { $sort = 'IPup'; }
	
	$ipsort = 'IPdown';
	$hnsort = 'HNdown';
	$insort = 'INdown';
	$ousort = 'OUdown';
	$insortM = 'INdownM';
	$ousortM = 'OUdownM';
	$insortW = 'INdownW';
	$ousortW = 'OUdownW';
	$insortD = 'INdownD';
	$ousortD = 'OUdownD';
	
	SWITCH: {
		if ($sort eq 'IPup') {
			$sortarrowip .= '&#9650;';
			$ipsort = 'IPdown';
			@sorted_ip = map {sprintf "%d.%d.%d.%d", split /[,.]/} sort map {sprintf "%03d.%03d.%03d.%03d", split /[,.]/} @ips;
			last SWITCH;
		}
		if ($sort eq 'IPdown') {
			$sortarrowip .= '&#9660;';
			$ipsort = 'IPup';
			@sorted_ip = map {sprintf "%d.%d.%d.%d", split /[,.]/} reverse sort map {sprintf "%03d.%03d.%03d.%03d", split /[,.]/} @ips;
			last SWITCH;
		}
		if ($sort eq 'HNup') {
			$sortarrowhn .= '&#9650;';
			$hnsort = 'HNdown';
			@sorted_ip = map { $_->[1] } sort { lc($a->[0]) cmp lc($b->[0]) } map { [ $traffic{$_}{'hostname'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'HNdown') {
			$sortarrowhn .= '&#9660;';
			$hnsort = 'HNup';
			@sorted_ip = map { $_->[1] } sort { lc($b->[0]) cmp lc($a->[0]) } map { [ $traffic{$_}{'hostname'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUup') {
			$sortarrowou .= '&#9650;';
			$ousort = 'OUdown';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'alloutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUdown') {
			$sortarrowou .= '&#9660;';
			$ousort = 'OUup';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'alloutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUupM') {
			$sortarrowouM .= '&#9650;';
			$ousortM = 'OUdownM';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'monthoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUdownM') {
			$sortarrowouM .= '&#9660;';
			$ousortM = 'OUupM';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'monthoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUupW') {
			$sortarrowouW .= '&#9650;';
			$ousortW = 'OUdownW';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'weekoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUdownW') {
			$sortarrowouW .= '&#9660;';
			$ousortW = 'OUupW';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'weekoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUupD') {
			$sortarrowouD .= '&#9650;';
			$ousortD = 'OUdownD';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'dayoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'OUdownD') {
			$sortarrowouD .= '&#9660;';
			$ousortD = 'OUupD';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'dayoutput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INup') {
			$sortarrowin .= '&#9650;';
			$insort = 'INdown';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'allinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INdown') {
			$sortarrowin .= '&#9660;';
			$insort = 'INup';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'allinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INupM') {
			$sortarrowinM .= '&#9650;';
			$insortM = 'INdownM';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'monthinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INdownM') {
			$sortarrowinM .= '&#9660;';
			$insortM = 'INupM';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'monthinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INupW') {
			$sortarrowinW .= '&#9650;';
			$insortW = 'INdownW';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'weekinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INdownW') {
			$sortarrowinW .= '&#9660;';
			$insortW = 'INupW';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'weekinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INupD') {
			$sortarrowinD .= '&#9650;';
			$insortD = 'INdownD';
			@sorted_ip = map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [ $traffic{$_}{'dayinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
		if ($sort eq 'INdownD') {
			$sortarrowinD .= '&#9660;';
			$insortD = 'INupD';
			@sorted_ip = map { $_->[1] } sort { $b->[0] <=> $a->[0] } map { [ $traffic{$_}{'dayinput'}, $_ ] } keys %traffic;
			last SWITCH;
		}
	}


	&Header::openbox('100%', 'left', "$Lang::tr{'IPTraffic_clientlist'}:");
	print" 
<style>
.table1colour {background-color: #d6d6d6;}
.table2colour {background-color: #f0f0f0;}
tr.table1colour:hover, tr.table2colour:hover { background: #a9a9a9; color: #fefefe;}
</style>
<table width='100%'>
	<tr bgcolor='#C0C0C0' style='heigt: 20px'>
		<td rowspan='2' width='30em' align='center'><b>#</b></td>
		<td rowspan='2' align='center'><b>$Lang::tr{'network'}</b></td>
		<td rowspan='2' width='110em' align='center'><b><a href='$ENV{'SCRIPT_NAME'}?SORT=$ipsort'>$Lang::tr{'ip address'}</a>".($sortarrowip eq '&nbsp;' ? '' : $sortarrowip)."</b></td>
		<td rowspan='2' width='110em' align='center'><b><a href='$ENV{'SCRIPT_NAME'}?SORT=$hnsort'>$Lang::tr{'hostname'}</a>".($sortarrowhn eq '&nbsp;' ? '' : $sortarrowhn)."</b></td>
		<td colspan='4' align='center'><b>$Lang::tr{'IPTraffic_load'}</b></td>
		<td colspan='4' align='center'><b>$Lang::tr{'IPTraffic_send'}</b></td>
		<td rowspan='2' colspan='3' align='center'><b>$Lang::tr{'action'}</b></td>
	</tr>
	<tr bgcolor='#C0C0C0'>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$insort'>$Lang::tr{'IPTraffic_sum'}</a>".($sortarrowin eq '&nbsp;' ? '' : $sortarrowin)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$insortM'>$Lang::tr{'IPTraffic_month'}</a>".($sortarrowinM eq '&nbsp;' ? '' : $sortarrowinM)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$insortW'>$Lang::tr{'IPTraffic_week'}</a>".($sortarrowinW eq '&nbsp;' ? '' : $sortarrowinW)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$insortD'>$Lang::tr{'IPTraffic_today'}</a>".($sortarrowinD eq '&nbsp;' ? '' : $sortarrowinD)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$ousort'>$Lang::tr{'IPTraffic_sum'}</a>".($sortarrowou eq '&nbsp;' ? '' : $sortarrowou)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$ousortM'>$Lang::tr{'IPTraffic_month'}</a>".($sortarrowouM eq '&nbsp;' ? '' : $sortarrowouM)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$ousortW'>$Lang::tr{'IPTraffic_week'}</a>".($sortarrowouW eq '&nbsp;' ? '' : $sortarrowouW)."</td>
		<td align='center' style='width: 6em'><a href='$ENV{'SCRIPT_NAME'}?SORT=$ousortD'>$Lang::tr{'IPTraffic_today'}</a>".($sortarrowouD eq '&nbsp;' ? '' : $sortarrowouD)."</td>
	</tr>";
	
		foreach my $ip (@sorted_ip) {
			if (!defined ($ip) || $ip eq '0.0.0.0') {next;}
			if (defined($traffic{$ip}{'limit'})) { 
				if (trim($traffic{$ip}{'limit'}) eq 'DEL') {next;}
			}
	
			if ( !defined ( $traffic{$ip}{'allinput'})) { $traffic{$ip}{'allinput'} = 0; }
			if ( !defined ( $traffic{$ip}{'alloutput'})) { $traffic{$ip}{'alloutput'} = 0; }
			if ( !defined ( $traffic{$ip}{'monthinput'})) { $traffic{$ip}{'monthinput'} = 0; }
			if ( !defined ( $traffic{$ip}{'monthoutput'})) { $traffic{$ip}{'monthoutput'} = 0; }
			if ( !defined ( $traffic{$ip}{'weekinput'})) { $traffic{$ip}{'weekinput'} = 0; }
			if ( !defined ( $traffic{$ip}{'weekoutput'})) { $traffic{$ip}{'weekoutput'} = 0; }
			if ( !defined ( $traffic{$ip}{'dayinput'})) { $traffic{$ip}{'dayinput'} = 0; }
			if ( !defined ( $traffic{$ip}{'dayoutput'})) { $traffic{$ip}{'dayoutput'} = 0; }
			
			$sum_incoming_all += $traffic{$ip}{'allinput'};
			$sum_outgoing_all += $traffic{$ip}{'alloutput'};
			$sum_incoming_month += $traffic{$ip}{'monthinput'};
			$sum_outgoing_month += $traffic{$ip}{'monthoutput'};
			$sum_incoming_week += $traffic{$ip}{'weekinput'};
			$sum_outgoing_week += $traffic{$ip}{'weekoutput'};
			$sum_incoming_day += $traffic{$ip}{'dayinput'};
			$sum_outgoing_day += $traffic{$ip}{'dayoutput'};

			my $netColor = getNetworkColor($ip);
			my $hostname = defined ($traffic{$ip}{'hostname'}) ? $traffic{$ip}{'hostname'} : ' ';

			print "
	<tr class='table".($count++ % 2?1:2)."colour'>
		<td align='center'>".$count."</td>
		<td align='center'><img src='$imgstatic/$netColor.png' alt='$netColor' title='$netColor' /></td>
		<td align='center'>$ip</td>
		<td align='center'>$hostname</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'allinput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'monthinput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'weekinput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'dayinput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'alloutput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'monthoutput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'weekoutput'})."</td>
		<td align='center'>".calcTraffic($traffic{$ip}{'dayoutput'})."</td>
		<td align='center' width='5%'>
			<form method='post' name='graph".$count."' action='$ENV{'SCRIPT_NAME'}'>
				<input type='hidden' name='ACTION' value='GRAPH' />
				<input type='hidden' name='PARAM' value='$ip' />
				<input type='image' name='$hostname' src='$imgstatic/graph.png' alt='$Lang::tr{'IPTraffic_graphs'}' title='$Lang::tr{'IPTraffic_graphs'}' />
			</form>
		</td>
		<td align='center' width='5%'>
			<form method='post' name='delete".$count."' action='$ENV{'SCRIPT_NAME'}'>
				<input type='hidden' name='ACTION' value='del' />
				<input type='hidden' name='PARAM' value='$ip' />
				<input type='image' name='$hostname' src='$imgsys/delete.gif' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' onclick=\"return confirm('$Lang::tr{'IPTraffic_delete_item'}')\"/>
			</form>
		</td>
	</tr>
		";
		}
		print "
	<tr bgcolor='#C0C0C0'>
		<td align='right' colspan='4' style='height: 2em'>$Lang::tr{'IPTraffic_sum'}</td>
		<td align='center'>".calcTraffic($sum_incoming_all)."</td>
		<td align='center'>".calcTraffic($sum_incoming_month)."</td>
		<td align='center'>".calcTraffic($sum_incoming_week)."</td>
		<td align='center'>".calcTraffic($sum_incoming_day)."</td>
		<td align='center'>".calcTraffic($sum_outgoing_all)."</td>
		<td align='center'>".calcTraffic($sum_outgoing_month)."</td>
		<td align='center'>".calcTraffic($sum_outgoing_week)."</td>
		<td align='center'>".calcTraffic($sum_outgoing_day)."</td>
		<td align='center' colspan='3'>&nbsp;</td>
	</tr>
</table>";
	
	&Header::closebox();
}

## arp table entries
sub disp_arp {
	my $output = `/sbin/ip neigh list`;
	my @clientlist = getIPfromTable();
	my $count = 0;

	foreach my $line (split(/\n/, $output)){
		if ($line =~ m/^(.*) dev ([^ ]+) lladdr ([0-9a-f:]*) (.*)$/ || $line =~ m/^(.*) dev ([^ ]+)  (.*)$/) {
			my $hostname = gethostbyaddr(inet_aton($1), AF_INET);
			$arpcache{$1} = $2.",".( $hostname ? $hostname : '' )."\n";
		}
	}

	&Header::openbox('100%', 'left', "$Lang::tr{'IPTraffic_add'}:");
	print"
<style>	.shownone { display: none; } </style>
<div>
<span style='float: left; background-color: #D6D6D6; padding: 0.8em;'><b>$Lang::tr{'IPTraffic_arp_table_entries'}</b></span>
<span style='float: right;'><button id='shownone'>$Lang::tr{'IPTraffic_view_table'}</button></span>
</div>
<div style='margin-top: 5em'>
<table width='100%' class='shownone' style='display: none;'>
<tr style='background: #C0C0C0;'>
	<td width='15%' align='center' height='20'><b>$Lang::tr{'network'}</b></td>
	<td width='20%' align='center' height='20'><b>$Lang::tr{'ip address'}</b></td>
	<td width='65%' align='center' height='20'><b>$Lang::tr{'hostname'}</b></td>
	<td width='5%' align='center' height='20'><b>$Lang::tr{'action'}</b></td>
</tr>
";
	foreach my $key (sort keys %arpcache) {
		chomp ($key);
		@line = split (/\,/, $arpcache{$key});
		my $netname = (split(/-/,$line[0]))[0];
		chop $netname;
		if ($netname eq 'red') {next;}
		chomp $line[1];
		print"
	<tr class='table".($count++ % 2?1:2)."colour' height='20'>
		<td align='center'><img src='$imgstatic/$netname.png' alt='$line[0]' title='$line[0]' /></td>
		<td align='center'>$key</td>
		<td align='center'>$line[1]</td>
		<td align='center'>";
		if (grep /^$key$/, @clientlist){
			print "
			<img src='$imgstatic/addfaint.gif' alt='$Lang::tr{'IPTraffic_no_add'}' title='$Lang::tr{'IPTraffic_no_add'}' />";
		}
		else {
			print "
			<form method='post' name='addfromlist".($idarp++)."' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
			<input type='hidden' name='ACTION' value='add' />
			<input type='hidden' name='PARAM' value='$key' />
			<input type='hidden' name='CLIENTNAME' value='$line[1]' />
			<input type='image' name='$netname' src='$imgstatic/add$netname.gif' alt='$Lang::tr{'IPTraffic_add'}' title='$Lang::tr{'IPTraffic_add'}' />
			</form>";
		}
		print "
		</td>
	</tr>";
	}

	print"
</table>
</div>
<script>
var shownone = false;
document.querySelector(\"#shownone\").onclick = function () {
	if (shownone === false ) {
		document.querySelectorAll(\".shownone\")[0].style=\"display: block\";
		shownone = true;
		this.innerHTML = \"$Lang::tr{'IPTraffic_hide_table'}\";
	} else {
		document.querySelectorAll(\".shownone\")[0].style=\"display: none\";
		shownone = false;
		this.innerHTML = \"$Lang::tr{'IPTraffic_view_table'}\";
	}
}
</script>
";

	&Header::closebox();
}

sub calcTraffic {
	my $value = shift;
	my @unit = ('', 'K', 'M', 'G', 'T', 'P');
	my $i = 0;

	while ($value > 1000){
		$value = sprintf ( "%.1f", $value/1024);
		$i++;
	}
	return ($value < 0 ? "0" : $value+0 )."&nbsp;$unit[$i]B";
}

sub setClientAction {
	my $client = shift;
	my $action = shift;
	my $host = shift;

	my $sql = '';
	if ($action eq 'ADD'){
		if (defined $host){
			$sql = "INSERT INTO '".$IPTrafficTable."' (`client`, `hostname`, `date`) VALUES ( '".$client."', '".$host."', date('now'));";
		}
		else {
			$sql = "INSERT INTO '".$IPTrafficTable."' (`client`, `date`) VALUES ( '".$client."', date('now'));";
		}
	}

	if ($action eq 'DEL'){
		$sql = "UPDATE '".$IPTrafficTable."' SET `traflimit` = 'DEL' WHERE `client` = '".$client."';";
	}
	my $dbh = DBI->connect("dbi:SQLite:dbname=$iptrafficdb", "", "", {RaiseError => 1, AutoCommit=>1});
	my $Action= $dbh->prepare($sql);
	$Action->execute();
	$Action->finish();
}

## get network color
sub getNetworkColor {
	my $ipaddress = shift;
	my %devs_color = ('GREEN' => 'green', 'BLUE' => 'blue', 'ORANGE' => 'orange', 'RED' => 'red', 'GRAY' => 'gray');

	for my $color (keys %devs_color) {
		my $interface = $devs_color{$color};
		if (exists($netsettings{$color."_DEV"})){
			next if ( $netsettings{$color."_DEV"} eq 'red0' && $netsettings{"RED_TYPE"} eq 'PPPOE');
	
			if ( &General::IpInSubnet($ipaddress, $netsettings{$color."_ADDRESS"}, $netsettings{$color."_NETMASK"}) ) {
				return $interface;
			}
		}
	}
	return 'red';
}
sub getHostnameFromTable {
	my $client = shift;
	my ($host, $sql);
	my $dbh = DBI->connect("dbi:SQLite:dbname=/var/log/iptraffic/iptraffic.db", "", "", {RaiseError => 1});
	$sql = "SELECT DISTINCT `hostname` FROM `".$IPTrafficTable."` WHERE `client` = '".$client."';";
	my $Statement = $dbh->prepare($sql);
	$Statement->execute;
	$Statement->bind_columns(\$host);
	$Statement->fetch();
	$Statement->finish();
	$dbh->disconnect() or warn $dbh->errstr;
	return $host;
}

sub getIPfromTable {
	my @clients = '';
	my ($client, $traflimit, $sql);
	my $dbh = DBI->connect("dbi:SQLite:dbname=/var/log/iptraffic/iptraffic.db", "", "", {RaiseError => 1});
	$sql = "SELECT DISTINCT `client`, `traflimit` FROM `".$IPTrafficTable."` WHERE `date` = date('now');";
	my $Statement = $dbh->prepare($sql);
	$Statement->execute;
	$Statement->bind_columns(\$client, \$traflimit);
	while($Statement->fetch()){
		$client = trim($client);
		if(length($client) > 0) {push(@clients, "$client");}
	}
	$Statement->finish();
	$dbh->disconnect() or warn $dbh->errstr;
	return @clients;
}

sub getAllTrafficFromTable {
	my %traffic;
	my @iplist = getIPfromTable();
	my ($client, $hostname, $date, $input, $output, $dayinput, $dayoutput, $weekinput, $weekoutput, $monthinput, $monthoutput, $traflimit, $port);

	my $dbh = DBI->connect("dbi:SQLite:dbname=/var/log/iptraffic/iptraffic.db", "", "", {RaiseError => 1});
	my $sql = '';
	my $Statement = $dbh->prepare($sql);
	foreach $client ( @iplist) {
		$client = trim($client);
		if (length($client) > 0){
			$sql = "SELECT `hostname`, sum(`input`), sum(`output`), ";
			$sql.= "(SELECT sum(input) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` = date('now')) as `dayinput`, ";
			$sql.= "(SELECT sum(output) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` = date('now')) as `dayoutput`, ";
			$sql.= "(SELECT sum(input) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` BETWEEN date('now', 'weekday 1', '-7 day') AND date('now')) as `weekinput`, ";
			$sql.= "(SELECT sum(output) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` BETWEEN date('now', 'weekday 1', '-7 day') AND date('now')) as `weekoutput`, ";
			$sql.= "(SELECT sum(input) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` BETWEEN date('now', 'start of month') AND date('now')) as `monthinput`, ";
			$sql.= "(SELECT sum(output) from '".$IPTrafficTable."' WHERE `client` = '".$client."' AND `date` BETWEEN date('now', 'start of month') AND date('now')) as `monthoutput`, ";
			$sql.= "`traflimit`, `port` from '".$IPTrafficTable."' WHERE `client` = '".$client."';";
			$Statement = $dbh->prepare($sql);
			$Statement->execute;
			$Statement->bind_columns(\$hostname, \$input, \$output, \$dayinput, \$dayoutput, \$weekinput, \$weekoutput, \$monthinput, \$monthoutput, \$traflimit, \$port);
			while($Statement->fetch()) {
				$traffic{$client} = {'hostname' => $hostname, 'allinput' => $input, 'alloutput' => $output, 'dayinput' => $dayinput, 'dayoutput' => $dayoutput, 'weekinput' => $weekinput, 'weekoutput' => $weekoutput, 'monthinput' => $monthinput, 'monthoutput' => $monthoutput, 'limit' => $traflimit, 'port' => $port};
			}
		}
	}
	$Statement->finish();
	$dbh->disconnect() or warn $dbh->errstr;

	return %traffic;
}

sub trim() {
	my $str = shift;
	if (defined $str) {$str =~ s/^\s+|\s+$//g;}
	return $str;
}
############################################################################################################################

sub hrline { 

print"<table width='100%'><tr><td colspan='2' height='35'><hr></td></tr></table>";

}

